//      Part of x2divx 
//      avi2divx An avi to divx ;-) encoder frontend to avifile
//      Copyright (C) 2000/10/23 Ulrich Hecht <uli@emulinks.de>
//      licensed under the GNU Public License (GPL) v2
//
//      This program is free software; you can redistribute it and/or modify
//      it under the terms of the GNU General Public License version 2, and
//      ONLY version 2, as published by
//      the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program; if not, write to the 
//	Free Software Foundation, Inc., 
//	59 Temple Place, Suite 330,
//	Boston, MA  02111-1307  USA

#define APPNAME "avi2divx"
#include "version.h"		// Contains APPVERSION AND LastChanged 
				// as it saves typing.
#include <iostream.h>		// I assume this is needed, not sure about C++
#include <videoencoder.h>
#include <avifile.h>
#include <aviplay.h>
#include <avifile/version.h>
#if AVIFILE_MAJOR_VERSION == 0 && AVIFILE_MINOR_VERSION < 50
#include <image.h>
#include <fourcc.h>
#else
#include <aviutil.h>
#endif
#include <avifile/except.h>
				// To be sure to get avifile version
				// header. Needed if some programmer
				// puts version.h
				// in the standard include directory.
				// Also allows local version.h
#include <math.h>		// M_PI is better than included PI constant
#include <sys/time.h>
#include <unistd.h>		// Needed for the access call to check
				// if file exists
#include <getopt.h>		// getopt
#include <stdint.h>		// standard integer types
#include <stdlib.h>		// standard library with integer
				// division but not ALL of it as sucky
				// info docs state
#include <inttypes.h>		// the rest of integer division types feh
#include <stdio.h>
#include <string.h>		// For strcmp, don't know why but
				// redsnot 7.1's gcc howls.
#define __MODULE__ APPNAME	// Needed for avifile exceptions

#if AVIFILE_MAJOR_VERSION == 0 && AVIFILE_MINOR_VERSION < 50
#include <creators.h>
#endif

static void
print_usage (void)
{
  printf ("\nUsage: %s [OPTION]... [input AVI]... -o [output AVI]\n",
	  APPNAME);
  exit (0);
}

static void
print_help (void)
{
  printf ("\nUsage: %s [OPTION]... [input AVI]... -o [output AVI]\n",
	  APPNAME);
  printf ("\nOptions:\n");
  printf ("  -b --divxbitrate\tDivX ;-) bitrate (default 700)\n");
  printf ("  -f --fast_motion\tuse fast-motion codec (default low-motion)\n");
#if AVIFILE_MAJOR_VERSION == 0 && AVIFILE_MINOR_VERSION < 50
  printf ("  -p --opendivx\t use OpenDivx codec\n");
#endif
  printf ("  -a --mp3bitrate\tMP3 bitrate (kBit, default auto)\n");
  printf ("  -s --startframe\tnumber of first frame to encode (default 0)\n");
  printf ("     --skipvideo\tnumber of video frames to skip (A/V sync)\n");
  printf ("  -e --endframe\t\tnumber of frames to encode (default all)\n");
  printf
    ("  -c --cliplines\tnumber of lines to clip from top/bottom (default none)\n");
  printf
    ("  -z --clipoffset\toffset of clipping window (default 0 = center)\n");
  printf ("  -l --blur\t\tradius for blur filter (default off)\n");
  printf ("  -w --width\t\toutput width (default input width)\n");
  printf ("  -h --height\t\toutput height (default input height)\n");
  printf ("  -o --outputfile\tOutput filename IS REQUIRED\n");
  printf ("  -v --version\t\tVersion and license.\n");
  printf
    ("     --forcedaudiorate\taudio sample rate of input file (Hz);\n\t\t\tuse only if avifile gets it wrong\n");
  //printf("  -g\tguess values for -c and -z options\n");
#if AVIFILE_MAJOR_VERSION == 0 && AVIFILE_MINOR_VERSION < 50
  printf ("  -k --keyframes\tset keyframes attribute (default 15)\n");
  printf ("  -C --crispness\tset crispness attribute (default 20)\n");
#endif
  printf
    ("  -u --flip_video\tflip picture (use if output video stream is upside-down,\n\t\t\tdefault off)\n");
  printf ("     --help\t\tPrint this help list.\n");
  printf ("\n" APPNAME " version " APPVERSION
	  " by Ulrich Hecht <uli@emulinks.de>\n");
  printf ("Last changed on " LastChanged "\n");
  exit (0);
}

static void
display_license (void)
{
  printf ("\nThis is %s version %s \n", APPNAME, APPVERSION);
  printf ("%s", "Copyright (C) Ulrich Hecht <uli@emulinks.de> \n\
This program is distributed in the hope that it will be useful,\n\
but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
GNU General Public License for more details.\n");
  exit (0);
}

int
main (int argc, char **argv)
{

  if (GetAvifileVersion () != AVIFILE_VERSION)
    {
      cout << "This binary was compiled for Avifile ver. " << AVIFILE_VERSION
	<< ", but the library is ver. " << GetAvifileVersion () <<
	". Aborting." << endl;
      return 0;
    }

  IAviReadFile *file;

  int copt;			// getopt switch variable
  imaxdiv_t divresults;		// A struct with memebers quot and 
  // rem which hold the output of imaxdev
  intmax_t framerate_cast;	// an intmax_t that holds a cast from 
  // the double framerate 
  intmax_t rate_cast;		// an intmax_t that holds a cast from int rate
  intmax_t achan_cast;		// an intmax_t that holds a cast from int achan
  intmax_t bpersamp_cast;	// bpersamp is wBitsPerSample

  char *outputfile = NULL;	// gcc howled at some point so I put it here
  int divxbitrate = 700;
  int mp3bitrate = -1;
#if AVIFILE_MAJOR_VERSION == 0 && AVIFILE_MINOR_VERSION < 50
  int codec = RIFFINFO_DIV3;
#else 
  int codec = fccDIV3;
#endif
  int endframe = -1;
  int cliplines = 0;
  int clipoffset = 0;
  int blur = 0;
  int owidth = 0;
  int oheight = 0;
  int forcedaudiorate = 0;
  int guess = 0;
  int flip = 1;
  int skipvframes = 0;
  int startframe = 0;
#if AVIFILE_MAJOR_VERSION == 0 && AVIFILE_MINOR_VERSION < 50
  int keyframes = 15;
  int crispness = 20;
#endif

  static char x1, x2;
  static unsigned int NewY1, NewX1, NewX2, Size, x, y, c;
  static double ScaleX, ScaleY, t1, t2, t4, f, ft;

  // double PI      = 3.14159265359f;    
  // M_PI is the long version in 
  // <math.h> and is more accurate

  // Ok I've redone this whole section for getopt
  // Pretty easy I had to change the options bit above 
  // To steal -o for outputfile and make all options one character
  // The snarly mess with the input files took a while and 
  // Could probably be improved on but this puts paid to
  // core dumps on non-existant or mistyped filenames
  // Since getopt howls when there is no arg to an option 
  // I didn't bother providing my own howling.

  while (1)
    {
      int option_index = 0;

      // Ok a few piddly long option but my gawd there has
      // to be a better way to parse this
      // This example weasels out by just using the
      // short options. Except for help.
      // For real horror, look at wgets options list
      // yea may the gawds smile upon those poor
      // sway backed, downtrodden, programmers.

      static struct option long_options[] = {
	{"help", 0, 0, 0},
	{"version", no_argument, NULL, 'v'},
	{"guess", no_argument, NULL, 'g'},
	{"license", no_argument, NULL, 'v'},
	{"divxbitrate", required_argument, NULL, 'b'},
	{"fast_motion", no_argument, NULL, 'f'},
	{"mp3bitrate", required_argument, NULL, 'a'},
	{"endframe", required_argument, NULL, 'e'},
	{"cliplines", required_argument, NULL, 'c'},
	{"clipoffset", required_argument, NULL, 'z'},
	{"blur", required_argument, NULL, 'l'},
	{"forcedaudiorate", required_argument, NULL, 0},
	{"width", required_argument, NULL, 'w'},
	{"height", required_argument, NULL, 'h'},
	{"audio_stream", required_argument, NULL, 'A'},
	{"video_stream", required_argument, NULL, 'V'},
	{"number_cpus", required_argument, NULL, 'n'},
	{"outputfile", required_argument, NULL, 'o'},
	{"flip_video", no_argument, NULL, 'u'},
	{"startframe", required_argument, NULL, 's'},
	{"skipvideo", required_argument, NULL, 0},
#if AVIFILE_MAJOR_VERSION == 0 && AVIFILE_MINOR_VERSION < 50
	{"keyframes", required_argument, NULL, 'k'},
	{"crispness", required_argument, NULL, 'C'},
	{"opendivx", no_argument, NULL, 'p'},
#endif
	{0, 0, 0, 0}
      };

      copt =
#if AVIFILE_MAJOR_VERSION == 0 && AVIFILE_MINOR_VERSION < 50
	getopt_long (argc, argv, "vgb:fa:e:c:z:l:w:h:A:V:n:o:u:s:k:C:p",
		     long_options, &option_index);
#else
	getopt_long (argc, argv, "vgb:fa:e:c:z:l:w:h:A:V:n:o:u:s:",long_options,
		     &option_index);
#endif
      if (copt == -1)
	break;

      switch (copt)
	{
	case 0:
	  if (strcmp (long_options[option_index].name, "help") == 0)
	    print_help ();
	  if (strcmp (long_options[option_index].name, "skipvideo") == 0)
	    skipvframes = atoi (optarg);
	  if (strcmp (long_options[option_index].name, "forcedaudiorate") ==
	      0)
	    forcedaudiorate = atoi (optarg);
	  break;

	case 'b':
	  divxbitrate = atoi (optarg);
	  break;

	case 'f':
	#if AVIFILE_MAJOR_VERSION == 0 && AVIFILE_MINOR_VERSION < 50
  	  codec = RIFFINFO_DIV4;
	#else 
  	  codec = fccDIV4;
	#endif
	  break;

	case 'a':
	  mp3bitrate = atoi (optarg);
	  break;

	case 's':
	  startframe = atoi (optarg);
	  break;

	case 'e':
	  endframe = atoi (optarg);
	  break;

	case 'c':
	  cliplines = atoi (optarg);
	  break;

	case 'z':
	  clipoffset = atoi (optarg);
	  break;

	case 'l':
	  blur = atoi (optarg);
	  break;

	case 'w':
	  owidth = atoi (optarg);
	  break;


	case 'h':
	  oheight = atoi (optarg);
	  break;

	case 'g':
	  guess = 1;
	  endframe = 100;
	  break;

	case 'o':
	  outputfile = optarg;
	  break;

#if AVIFILE_MAJOR_VERSION == 0 && AVIFILE_MINOR_VERSION < 50
	case 'k':
	  keyframes = atoi (optarg);
	  break;

	case 'C':
	  crispness = atoi (optarg);
	  break;

	case 'p':
	  codec = RIFFINFO_DIVX;
	  break;
#endif

	case 'u':
	  flip = 0;
	  break;

	case 'v':
	  display_license ();
	  break;

	case '?':
	  print_usage ();
	  break;

	case ':':
	  printf ("You missed giving something and argument.\n");	
				// since we have non-options
	  break;		// never gets called
	}
    }

  int numfiles = argc - optind;
  if (numfiles <= 0)
    {
      printf
	("\nI count the number of input files as %i.\nDid you input a file?\n",
	 numfiles);
      print_usage ();
    }

  int tempor;
  char *inputfiles[numfiles];
  int i = 0;

  for (; optind < argc; optind++)
    {

      inputfiles[i] = argv[optind];

      tempor = access (inputfiles[i], R_OK);
      if (tempor < 0)
	{
	  printf
	    ("Sorry there is a problem with the file %s you specified.\n",
	     inputfiles[i]);
	  printf ("Please try again. \n");
	  exit (1);
	}
      ++i;
    }

  // check on output filename here since it 
  // stops a lot of avifile spewing.
  // could also check if file exists and 
  // not allow overwrite...could  
  // add a switch to allow overwrite...nah

  if (outputfile == NULL)
    {
      printf ("\nOutput filename IS REQUIRED use -o filename\n");
      exit (1);
    }

  try
  {
    file = CreateIAviReadFile (inputfiles[0]);
  }

  // Ok I can catch a fatal error here but it doesn't catch other
  // errors later. This does stop cores on invalid AVI files though.
  // Unfortunately without opening and closing each file in the file
  // checking loop above as with mpeg2divx all you are told here is that
  // something crashed but not which file. I suppose it would be
  // possible to open and close each file to check but, sheesh that's 
  // a lot of work and eats a lot of precious time.

  catch (FatalError & error)
  {
    printf ("\nThere is an error with one of the input avifiles: \n");
    error.PrintAll ();
    exit (1);
    // Exit never gets called as far as I can tell.
  }

  IAviReadStream *invstream = file->GetStream (0, AviStream::Video);

  BITMAPINFOHEADER bh;
  invstream->StartStreaming ();
  invstream->GetDecoder ()->SetDestFmt (24);
  invstream->GetDecoder ()->SetDirection (flip);
  invstream->GetOutputFormat (&bh, sizeof bh);
  int origwidth = bh.biWidth = abs (bh.biWidth);
  int origheight = bh.biHeight = abs (bh.biHeight);

  if (oheight || owidth)
    {
      if (oheight)
	bh.biHeight = oheight;
      if (owidth)
	bh.biWidth = owidth;
    }

  bh.biHeight -= cliplines * 2;
  int width = bh.biWidth;
  int height = bh.biHeight;

  unsigned char *framebuffer = new unsigned char[width * height * 3];
  CImage imtarget ((BitmapInfo *) & bh, framebuffer, false);

  double framerate = 1 / invstream->GetFrameTime ();
  // printf("framerate: %d\n",framerate);

#if AVIFILE_MAJOR_VERSION == 0 && AVIFILE_MINOR_VERSION < 50
  int frames = invstream->GetLength ();
#else
  int frames = invstream->GetEndPos ();
#endif


  // I believe this causes the segfault I mentioned in the TODO
  // An mpeg encoded with achan <= 0 will cause a failure if
  // you do mpeg->divx->divx . I believe this means that an avi
  // without an audio track will fail. This requires a complete 
  // rewrite I think after looking at qtrecompress using GetType. 
  // I am NOT that good at C++

  // Well guess I can sort of code, this "fixes" segfault on 
  // Audioless AVIs by refusing further processing. I blead
  // the source code to qtrecompress but stopped since the
  // thing segfaults no matter what and it looked painful
  // to attempt anything in that QuTe mess.

  // Later...And I came up with a somewhat useful solution.
  // I use audioexist as a toggle for whether or not
  // inastream grabbed an audio stream and use that
  // as a flag. It will now convert my soundless mpegs
  // flawlessly.

  IAviReadStream *inastream = file->GetStream (0, AviStream::Audio);

  WAVEFORMATEX fmt;		// Had to be moved here

  // After writing this I notice it's almost identical to mpeg2divx's
  // code. Other than using inastream here to do the switching.

  int audioexist = 1;

  if (inastream == NULL)
    {
      audioexist = 0;
    }
  else
    {
      inastream->StartStreaming ();
      inastream->GetOutputFormat (&fmt, sizeof fmt);
    }

  int achan = fmt.nChannels;
  int rate = fmt.nSamplesPerSec;
  int bpersamp = fmt.wBitsPerSample;

  if (audioexist <= 0)
    {
      achan = 0;
      rate = 0;
      bpersamp = 0;
    }

  if (forcedaudiorate)
    {
      fmt.nSamplesPerSec = rate = forcedaudiorate;
    }

  printf ("achan %d, rate %d\n", achan, rate);

  //unsigned char* framebuffer = new unsigned char[width*height*3];

  // Hopefull the following hairball fixes  
  // gcc howling about dividing a double 
  // by an int. Hrumph not quite worth the effort 
  // except I found out that to use some of the more 
  // interesting interger math routines you include 
  // inttypes.h which of course NO one bothers to 
  // mention anywhere. jpc

  int sampsperframe;
  // Declare this to be an integer -modified code-

  framerate_cast = (intmax_t) framerate;
  // cast framerate to intmax integer -new code-

  rate_cast = (intmax_t) rate;
  // cast rate to intmax integer -new code-

  divresults = imaxdiv (rate_cast, framerate_cast);
  // Using integer math divide the two, this seems to 
  // round up one more than the orignial int 
  // division of a double and GCC likes it. -new code-

  sampsperframe = divresults.quot;
  // Store the intmax result back in sampsperframe, 
  // this does not make gcc howl but divresults.quot 
  // is a intmax_t type. -new code-

  // printf("rate and framerate cast %lli %lli\n", rate_cast,
  // framerate_cast); 
  // print it out for reference -new code-

  printf ("---sampsperframe: %d\n", sampsperframe);
  // print it out again 

  short *audiobufferint = new short[sampsperframe * 2];


  int fccHandler = codec;
  IAviWriteFile *avifile;

  avifile = CreateIAviWriteFile (outputfile);

#if AVIFILE_MAJOR_VERSION == 0 && AVIFILE_MINOR_VERSION < 50
  const CodecInfo *codecInfo = CodecInfo::match (codec);
  if (codec != RIFFINFO_DIVX)
    {
      Creators::SetCodecAttr (*codecInfo, "BitRate", divxbitrate);
      Creators::SetCodecAttr (*codecInfo, "Crispness", crispness);
      Creators::SetCodecAttr (*codecInfo, "KeyFrames", keyframes);
    }
#else
  IVideoEncoder::SetExtendedAttr (codec, "BitRate", divxbitrate);
  //IVideoEncoder::SetExtendedAttr(codec, "Crispness",99);
#endif

  IAviVideoWriteStream *stream = avifile->AddVideoStream (fccHandler, &bh,
							  (int) (invstream->
								 GetFrameTime
								 () *
								 1000000));
  // The formatting above was done by indent. 
  // Wierd looking but supposedly GNU standard.
  // The added (int) above casts the float result 
  // GetFrameTime back to int this is sane when 
  // GetFrameTime() is multiplied by 1 million. -modified code-

  // printf("This is GetFrameTime() times 1 million then cast to int:  %i\n", 
  // (int)(invstream->GetFrameTime()*1000000));
  // printf("This is 1/GetFrameTime():  %f cast to int %i \n", 
  // 1/(invstream->GetFrameTime()),  (int)(1/(invstream->GetFrameTime())));
  // Commented out debugging printfs.

  stream->SetQuality (8000);

  stream->Start ();

  if (mp3bitrate == -1)
    switch (rate)
      {
      case 48000:
	mp3bitrate = 80 * achan;
	break;
      case 44100:
	mp3bitrate = 56 * achan;
	break;
      case 22050:
	mp3bitrate = 32 * achan;
	break;
      default:
	if (audioexist <= 0)
	  {
	    printf
	      ("Audio doesn't appear to exist so no audio will be converted\n");
	    break;
	  }
      }
  IAviAudioWriteStream *astream = 0;

  if (audioexist > 0)
    {
      astream = avifile->AddAudioStream (0x55, &fmt, (mp3bitrate * 1000) / 8);
      astream->Start ();
    }

  Unsigned samples_read = 0;
  Unsigned bytes_read = 0;

  int totalframes = 0;
  int totalframes_allfiles = frames;

  struct timeval tv;
  struct timezone tz = { 0, 0 };
  gettimeofday (&tv, &tz);
  long startsec = tv.tv_sec;
  long startusec = tv.tv_usec;
  double fps = 0;

  // The following does the same for audiobytesperframe that I earlier
  // did for sampsperframe. It also rounds up by one, which seems to be
  // a good thing.

  int audiobytesperframe;
  // -modified code-

  rate_cast = (intmax_t) rate;	//-new code-
  achan_cast = (intmax_t) achan;	//-new code-
  bpersamp_cast = (intmax_t) bpersamp;	//-new code-
  framerate_cast = (intmax_t) framerate;	//-new code-

  // printf("rate,achan,bpersamp,framerate %lli %lli %lli %lli\n", 
  // rate_cast, achan_cast, bpersamp_cast, framerate_cast);
  // The above is just a debug printfs.. -new code-
  divresults = imaxdiv (bpersamp_cast, 8);	//-new code-
  bpersamp_cast = divresults.quot;
  // I got tired of typing variables -new code- 

  bpersamp_cast *= rate_cast;
  // So I just reused one -new code-

  bpersamp_cast *= achan_cast;
  // It seems to work ok -new code-

  bpersamp_cast /= framerate_cast;
  //This is the same as multiplying by GetFrameTime
  // Ha. -new code-

  audiobytesperframe = (intmax_t) bpersamp_cast;

  // Ok I did it, and it seems to do the math right and gcc licks it.      
  // I probably have broken some rule here. I've tried this with 
  // A potload of different frame rates and mono/stereo audio
  // Seems solid. -new code-

  printf ("audiobytesperframe: %d\n", audiobytesperframe);
  int written_audio = 0;

  int lineguesstop = 0;
  int lineguessbottom = 0;
  int lineguesstopcount = 0;
  int lineguessbottomcount = 0;

  for (int j = 0; j < numfiles; j++)
    {
      if (j)
	{
	  delete file;
	  file = CreateIAviReadFile (inputfiles[j]);

	  invstream = file->GetStream (0, AviStream::Video);

	  invstream->StartStreaming ();
	  invstream->GetDecoder ()->SetDestFmt (24);
	  invstream->GetDecoder ()->SetDirection (flip);

#if AVIFILE_MAJOR_VERSION == 0 && AVIFILE_MINOR_VERSION < 50
	  frames = invstream->GetLength ();
#else
	  frames = invstream->GetEndPos ();
#endif
	  totalframes_allfiles += frames;
	  inastream = file->GetStream (0, AviStream::Audio);

	  inastream->StartStreaming ();
	}

      if (startframe > 0)
	{
	  frames -= startframe;
	  totalframes_allfiles -= startframe;
	  printf ("Starting at frame %d\n", startframe);
	  invstream->Seek (startframe);
	  inastream->SeekTime (invstream->GetTime ());
	  startframe = 0;
	}
      while (skipvframes > 0)
	{
	  invstream->ReadFrame ();
	  skipvframes--;
	}

      for (int i = 0; i < frames; i++, totalframes++)
	{
	  if (endframe > 0 && totalframes >= endframe)
	    goto finished;

	//if(totalframes%10 == 0) printf("Encoding frame
	//%d...\n",totalframes);
	  gettimeofday (&tv, &tz);
	  fps =
	    totalframes / ((tv.tv_sec + tv.tv_usec / 1000000.0) -
			   (startsec + startusec / 1000000.0));

	  if (!guess)
	    {
	      printf ("\rEncoding frame %d of %d, %.1f frames per sec    ",
		      totalframes, totalframes_allfiles, fps);
	      fflush (stdout);
	    }

	  if (skipvframes >= 0)
	    invstream->ReadFrame ();
	  else
	    skipvframes++;


	  CImage *imsrc = invstream->GetFrame ();

	  if (blur)
	    {
	      //imsrc->ToYUV();
	      imsrc->Blur (blur);
	      //imsrc->ToRGB();
	    }

	  if (!(oheight || owidth))
#if AVIFILE_MAJOR_VERSION == 0 && AVIFILE_MINOR_VERSION < 50
	    memcpy (framebuffer,
		    imsrc->Data () + (cliplines - clipoffset) * width * 3,
		    width * height * 3);
#else
	    memcpy (framebuffer,
		    imsrc->data () + (cliplines - clipoffset) * width * 3,
		    width * height * 3);
#endif
	  else
	    {
	      ScaleX = (float) bh.biWidth / origwidth;
	      ScaleY = (float) bh.biHeight / origheight;

	      for (y = 0; abs ((int) y) < abs (bh.biHeight); y++)	//obscure gcc error without the abs jpc
		{
		  NewY1 =
		    (unsigned int) (y / ScaleY + cliplines -
				    clipoffset) * /*Image->Bps */ origwidth *
		    3;
		  for (x = 0; abs ((int) x) < abs (bh.biWidth); x++)	//obscure gcc error without the abs jpc
		    {
		      t1 = x / (double) bh.biWidth;
		      t4 = t1 * bh.biWidth;
		      t2 = t4 - (unsigned int) (t4);
		      ft = t2 * M_PI;
		      f = (1.0 - cos (ft)) * .5;
		      NewX1 = ((unsigned int) (t4 / ScaleX)) * 3;
		      NewX2 = ((unsigned int) (t4 / ScaleX) + 1) * 3;

		      Size = y * bh.biWidth * 3 /*Scaled->Bps */  + x * 3;
		      for (c = 0; c < 3; c++)
			{
#if AVIFILE_MAJOR_VERSION == 0 && AVIFILE_MINOR_VERSION < 50
			  x1 = imsrc->Data ()[NewY1 + NewX1 + c];
			  x2 = imsrc->Data ()[NewY1 + NewX2 + c];
#else
			  x1 = imsrc->data ()[NewY1 + NewX1 + c];
			  x2 = imsrc->data ()[NewY1 + NewX2 + c];
#endif
			  //printf ("X1: %d, X2: %d, Size: %d\n", NewY1 + NewX1 + c, NewY1 + NewX2 + c, width * height *3);
			  framebuffer[Size + c] =
			    (unsigned char) ((1.0 - f) * x1 + f * x2);
			  //printf("Scale: %d, Size: %d\n", Size + c, bh.biWidth*bh.biHeight *3 );
			}
		    }
		}
	    }

	  delete imsrc;

	  if (guess)
	    {
	      double totallum;
	      double oldlum = 0;
	      for (int y = 0; y < height; y++)
		{
		  totallum = 0;
		  for (int x = 0; x < width * 3; x++)
		    {
		      totallum += *(framebuffer + y * width * 3 + x);
		    }
		  totallum /= 3 * width;
		//printf("line %d: totallum: %lf, oldlum:
		//%lf\n",y,totallum,oldlum);
		  if (totallum > oldlum + 30)
		    {
		      printf ("top line: %d\n", y);
		      lineguesstop += y;
		      lineguesstopcount++;
		      break;
		    }
		  oldlum = totallum;
		}
	      for (int y = height; y > 0; y--)
		{
		  totallum = 0;
		  for (int x = 0; x < width * 3; x++)
		    {
		      totallum += *(framebuffer + y * width * 3 + x);
		    }
		  totallum /= 3 * width;
		  if (totallum > oldlum + 30)
		    {
		      printf ("bottom line: %d\n", y);
		      lineguessbottom += y;
		      lineguessbottomcount++;
		      break;
		    }
		  oldlum = totallum;
		}
	      continue;
	    }

	  stream->AddFrame (&imtarget);

	  long long excess =
	    (long long) (audiobytesperframe * (totalframes + 1) -
			 written_audio);
	  while ((excess > 0) && (!inastream->Eof ()))
	    {
	      inastream->ReadFrames (audiobufferint,
				     (excess >
				      sampsperframe * achan *
				      2) ? excess : sampsperframe * achan * 2,
				     sampsperframe * achan * 2, samples_read,
				     bytes_read);
	      written_audio += bytes_read;
	      excess -= bytes_read;
		//printf("samples_read %d, bytes_read
		//%d\n",samples_read,bytes_read);
	      astream->AddData (audiobufferint, bytes_read);
	    }

	//printf("framebuffer = %d %d
	//%d\n",framebuffer[0],framebuffer[1],framebuffer[2]);
	//printf("Writing frame %d...\n",i);

	}
    }

finished:
  stream->Stop ();
  delete avifile;
  delete file;

  if (guess)
    {
      int avgtop = lineguesstop / lineguesstopcount;
      int avgbottom = lineguessbottom / lineguessbottomcount;
      printf ("avg line top: %f\n",
	      (double) lineguesstop / lineguesstopcount);
      printf ("avg line bottom: %f\n",
	      (double) lineguessbottom / lineguessbottomcount);
      printf ("suggested options: -c %d -o %d\n",
	      (avgtop + height - avgbottom) / 2,
	      -avgtop + ((avgtop + height - avgbottom) / 2));
    }
}
