//      Part of x2divx 
//      mpeg2divx
//      Copyright (C) 2000/10/23 Ulrich Hecht <uli@emulinks.de>
//      licensed under the GNU Public License (GPL) v2
//
//      This program is free software; you can redistribute it and/or modify
//      it under the terms of the GNU General Public License version 2, and
//      ONLY version 2, as published by
//      the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program; if not, write to the Free Software
//      Foundation, Inc., 
//	59 Temple Place, Suite 330, 
//	Boston, MA 02111-1307  USA

#define APPNAME "mpeg2divx"
#include "version.h"		// Contains APPVERSION AND LastChanged
				// as it saves typing.
#include <iostream.h>		// I assume this is needed, not sure
				// about C++
#include <videoencoder.h>
#include <avifile.h>
#include <aviplay.h>
#include <avifile/version.h>
#if AVIFILE_MAJOR_VERSION == 0 && AVIFILE_MINOR_VERSION < 50
#include <image.h>
#include <fourcc.h>
#else
#include <aviutil.h>
#endif
#include <avifile/except.h>
				// To be sure to get avifile version header. 
				// Needed if some programmer puts version.h
				// in the standard include directory.
				// Also allows local version.h
#include <libmpeg3.h>
#include <math.h>		// M_PI is better than included PI constant
#include <sys/time.h>
#include <unistd.h>		// Needed for the access call to check
				// if file exists
#include <getopt.h>		// getopt
#include <stdint.h>		// standard integer types
#include <stdlib.h>		// standard library with integer division
				// but not ALL of it as sucky info docs state
#include <inttypes.h>		// the rest of integer division types
#include <stdio.h>
#include <string.h>
#if AVIFILE_MAJOR_VERSION == 0 && AVIFILE_MINOR_VERSION < 50
#include <creators.h>
#endif

#define __MODULE__ APPNAME	// Needed for avifile exceptions



static void
print_usage (void)
{
  printf ("\nUsage: %s [OPTION]... [input MPEG]... -o [output AVI]\n",
	  APPNAME);
  exit (0);
}

static void
print_help (void)
{
  printf ("\nUsage: %s [OPTION]... [input MPEG]... -o [output AVI]\n",
	  APPNAME);
  printf ("\nOptions:\n");
  printf ("  -b --divxbitrate\tDivX ;-) bitrate (default 700)\n");
  printf ("  -f --fast_motion\tuse fast-motion codec (default low-motion)\n");
  printf ("  -p --opendivx\t\tuse OpenDivx codec\n");
  printf ("  -a --mp3bitrate\tMP3 bitrate (kBit, default auto)\n");
  printf ("  -s --startframe\tnumber of first frame to encode (default 0)\n");
  printf ("     --skipvideo\tnumber of video frames to skip (A/V sync)\n");
  printf ("  -e --endframe\t\tnumber of frames to encode (default all)\n");
  printf
    ("  -c --cliplines\tnumber of lines to clip from top/bottom (default none)\n");
  printf
    ("  -z --clipoffset\toffset of clipping window (default 0 = center)\n");
  printf ("  -l --blur\t\tradius for blur filter (default off)\n");
  printf ("  -w --width\t\toutput width (default input width)\n");
  printf ("  -h --height\t\toutput height (default input height)\n");
  printf ("  -n --number_cpus\tnum cpus for decoding (default 1)\n");
  printf ("  -A --audio_stream\taudio stream number (default 0)\n");
  printf ("  -V --video_stream\tvideo stream number (default 0)\n");
#if AVIFILE_MAJOR_VERSION == 0 && AVIFILE_MINOR_VERSION < 50
  printf ("  -k --keyframes\tset keyframes attribute (default 15)\n");
  printf ("  -C --crispness\tset crispness attribute (default 20)\n");
#endif
  printf ("  -o --outputfile\tOutput filename IS REQUIRED\n");
  printf ("  -v --version\t\tVersion and license.\n");
  printf ("     --help\t\tPrint this help list.\n");
  printf ("\n" APPNAME " version " APPVERSION
	  " by Ulrich Hecht <uli@emulinks.de>\n");
  printf ("Last changed on " LastChanged "\n");
  exit (0);
}

static void
display_license (void)
{
  printf ("\nThis is %s version %s \n", APPNAME, APPVERSION);
  printf ("%s", ("Copyright (C) 2000 Ulrich Hecht <uli@emulinks.de> \n\
This program is distributed in the hope that it will be useful,\n\
but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
GNU General Public License for more details.\n"));
  exit (0);
}

int
main (int argc, char **argv)
{

  if (GetAvifileVersion () != AVIFILE_VERSION)
    {
      cout << "This binary was compiled for Avifile ver. " << AVIFILE_VERSION
	<< ", but the library is ver. " << GetAvifileVersion () <<
	". Aborting." << endl;
      return 0;
    }

  mpeg3_t *file;
  int copt;			// getopt counter
  imaxdiv_t divresults;		// A struct with memebers quot
				// and rem which hold the output of
				// imaxdev.
  intmax_t framerate_cast;	// an intmax_t that holds a cast from 
  				// the double framerate
  intmax_t rate_cast;		// an intmax_t that holds a cast from
				// int of rate
  char *outputfile = NULL;	// gcc howled at some point so I put
				// it here
  int divxbitrate = 700;
  int mp3bitrate = -1;
#if AVIFILE_MAJOR_VERSION == 0 && AVIFILE_MINOR_VERSION < 50
  int codec = RIFFINFO_DIV3;
#else
  int codec = fccDIV3;
#endif
  int endframe = -1;
  int cliplines = 0;
  int clipoffset = 0;
  int blur = 0;
  int owidth = 0;
  int oheight = 0;
  int numcpus = 1;
  int austream = 0;
  int vistream = 0;
  int skipvframes = 0;
  int startframe = 0;
#if AVIFILE_MAJOR_VERSION == 0 && AVIFILE_MINOR_VERSION < 50
  int keyframes = 15;
  int crispness = 20;
#endif

  static char x1, x2;
  static unsigned int NewY1, NewX1, NewX2, Size, x, y, c;
  static double ScaleX, ScaleY, t1, t2, t4, f, ft;

  // NewY2, t3, NewX are unused 

  //double PI      = 3.14159265359f; 
  //M_PI is the long version in
  //math.h and is more accurate

  // Ok I've redone this whole section for getopt
  // Pretty easy I had to change the options bit above 
  // to steal -o for outputfile and make all options one character
  // The snarly mess with the input files took a while and 
  // Could probably be improved on but this puts paid to
  // core dumps on non-existant or mistyped filenames
  // Since getopt howls when there is no arg to an option 
  // I didn't bother providing my own howling.

  while (1)
    {
      int option_index = 0;

      // Ok a few piddly long option but my gawd there has
      // to be a better way to parse this
      // This example weasels out by just using the
      // short options. Except for help.
      // For real horror, look at wgets options list
      // yea may the gawds smile upon those poor
      // sway backed, downtrodden, programmers.

      static struct option long_options[] = {

	{"help", 0, 0, 0},
	{"version", no_argument, NULL, 'v'},
	{"guess", no_argument, NULL, 'g'},
	{"license", no_argument, NULL, 'v'},
	{"divxbitrate", required_argument, NULL, 'b'},
	{"fast_motion", no_argument, NULL, 'f'},
	{"mp3bitrate", required_argument, NULL, 'a'},
	{"endframe", required_argument, NULL, 'e'},
	{"cliplines", required_argument, NULL, 'c'},
	{"clipoffset", required_argument, NULL, 'z'},
	{"blur", required_argument, NULL, 'l'},
	{"width", required_argument, NULL, 'w'},
	{"height", required_argument, NULL, 'h'},
	{"audio_stream", required_argument, NULL, 'A'},
	{"video_stream", required_argument, NULL, 'V'},
	{"number_cpus", required_argument, NULL, 'n'},
	{"outputfile", required_argument, NULL, 'o'},
	{"startframe", required_argument, NULL, 's'},
	{"skipvideo", required_argument, NULL, 0},
#if AVIFILE_MAJOR_VERSION == 0 && AVIFILE_MINOR_VERSION < 50
	{"keyframes", required_argument, NULL, 'k'},
	{"crispness", required_argument, NULL, 'C'},
	{"opendivx", no_argument, NULL, 'p'},
#endif
	{0, 0, 0, 0}
      };

      copt =
#if AVIFILE_MAJOR_VERSION == 0 && AVIFILE_MINOR_VERSION < 50
	getopt_long (argc, argv, "vgb:fa:e:c:z:l:w:h:A:V:n:o:s:k:C:p",
		     long_options, &option_index);
#else
	getopt_long (argc, argv, "vgb:fa:e:c:z:l:w:h:A:V:n:o:s:",
		     long_options, &option_index);
#endif
      if (copt == -1)
	break;

      switch (copt)
	{
	case 0:
	  if (long_options[option_index].name == "help")
	    print_help ();
	  if (strcmp (long_options[option_index].name, "skipvideo") == 0)
	    skipvframes = atoi (optarg);
	  break;

	case 'b':
	  divxbitrate = atoi (optarg);
	  break;

	case 'f':
	#if AVIFILE_MAJOR_VERSION == 0 && AVIFILE_MINOR_VERSION < 50
  	  codec = RIFFINFO_DIV4;
	#else
  	  codec = fccDIV4;
	#endif
	  break;

	case 'a':
	  mp3bitrate = atoi (optarg);
	  break;

	case 's':
	  startframe = atoi (optarg);
	  break;

	case 'e':
	  endframe = atoi (optarg);
	  break;

	case 'c':
	  cliplines = atoi (optarg);
	  break;

	case 'z':
	  clipoffset = atoi (optarg);
	  break;

	case 'l':
	  blur = atoi (optarg);
	  break;

	case 'w':
	  owidth = atoi (optarg);
	  break;

	case 'h':
	  oheight = atoi (optarg);
	  break;

	case 'A':
	  austream = atoi (optarg);
	  break;

	case 'V':
	  vistream = atoi (optarg);
	  break;

	case 'n':
	  numcpus = atoi (optarg);
	  break;

#if AVIFILE_MAJOR_VERSION == 0 && AVIFILE_MINOR_VERSION < 50
	case 'k':
	  keyframes = atoi (optarg);
	  break;

	case 'C':
	  crispness = atoi (optarg);
	  break;

	case 'p':
	  codec = RIFFINFO_DIVX;
	  break;
#endif

	case 'o':
	  outputfile = optarg;
	  break;

	case 'v':
	  display_license ();
	  break;

	case '?':
	  print_usage ();
	  break;

	case ':':
	  printf ("You missed giving something and argument.\n");	
	// never gets called
	  break;		// cause we have non-options
	}
    }

  int numfiles = argc - optind;
  if (numfiles <= 0)
    {
      printf
	("\nI count the number of input files as %i.\nDid you input a file?\n",
	 numfiles);
      print_usage ();
      exit (1);
    }

  int tempor;
  char *inputfiles[numfiles];
  int i = 0;

  for (; optind < argc; optind++)
    {

      inputfiles[i] = argv[optind];

      tempor = mpeg3_check_sig (inputfiles[i]);
      if (tempor != 1)
	{
	  printf ("\nInvalid mpeg header on file %s .\n\n", inputfiles[i]);
	  exit (1);
	}
      ++i;
    }

  // check on output filename here since it 
  // stops a lot of avifile spewing.
  // could also check if file exists and 
  // not allow overwrite...could  
  // add a switch to allow overwrite...nah

  if (outputfile == NULL)
    {
      printf ("\nOutput filename IS REQUIRED use -o filename\n");
      exit (1);
    }

  // Since libmpeg3 is used to open the file(s) there is no other
  // checking done. I didn't see any other error handling
  // in libmpeg, I assume it's there for errors in the middle
  // of an mpeg.

  file = mpeg3_open (inputfiles[0]);

  int astrm = mpeg3_total_astreams (file);
  austream = (austream > astrm - 1 ? astrm - 1 : austream);
  printf ("set audio stream %d of %d\n", austream, astrm);

  int vstrm = mpeg3_total_vstreams (file);
  vistream = (vistream > vstrm - 1 ? vstrm - 1 : vistream);
  printf ("set video stream %d of %d\n", vistream, vstrm);

  int achan = mpeg3_audio_channels (file, austream);
  int rate = mpeg3_sample_rate (file, austream);
  long samples = mpeg3_audio_samples (file, austream);
  printf ("channels %i, rate %i, samples %li\n", achan, rate, samples);	// samples needs long int format

  mpeg3_set_cpus (file, numcpus);

  int width = mpeg3_video_width (file, vistream);
  int height = mpeg3_video_height (file, vistream);
  float framerate = mpeg3_frame_rate (file, vistream);
  int frames = mpeg3_video_frames (file, vistream);
  printf ("width %d, height %d, framerate %f, frames %d\n", width, height,
	  framerate, frames);

  unsigned char *framebuffer = new unsigned char[width * height * 3];
  unsigned char *extrabuffer = new unsigned char[width * 3 + 4];
  unsigned char **rowptr = new (unsigned char *)[height];

  for (int i = 0; i < height; i++)
    {
      if (i == height - 1)
	{
	  rowptr[i] = extrabuffer;
	}
      rowptr[i] = framebuffer + (width * (height - 1) * 3) - (width * 3 * i);
    }

  height -= cliplines * 2;

  //printf("framebuffer: %d",(unsigned int)framebuffer);

  // Hopefull the following hairball fixes  
  // gcc howling about dividing a double 
  // by an int. Hrumph not quite worth the effort 
  // except I found out that to use some of the more 
  // interesting interger math routines you include 
  // inttypes.h which of course NO one bothers to 
  // mention anywhere. jpc

  int sampsperframe;
  // Declare this to be an integer

  framerate_cast = (intmax_t) framerate;
  // cast framerate to intmax integer

  rate_cast = (intmax_t) rate;
  // cast rate to intmax integer

  divresults = imaxdiv (rate_cast, framerate_cast);
  // Using integer math divide the two, this seems to
  // round up one more that the orignial causes-gcc-to howl
  // int division of a double.

  sampsperframe = divresults.quot;
  // Store the intmax result back in sampsperframe, 
  // this does not make gcc howl
  // but divresults.quot is a intmax_t type.

  //printf("rate and framerate cast %lli %lli\n", rate_cast, framerate_cast); 
  //The above is commented out, I used it for testing, it is interesting to 
  //look at...once. 

  short *audiobufferint = new short[sampsperframe * 2];
  short *read_buffer = new short[sampsperframe];

  int fccHandler = codec;

  IAviWriteFile *avifile;

  avifile = CreateIAviWriteFile (outputfile);

#if AVIFILE_MAJOR_VERSION == 0 && AVIFILE_MINOR_VERSION < 50
  const CodecInfo *codecInfo = CodecInfo::match (codec);
  if (codec != RIFFINFO_DIVX)
    {
      Creators::SetCodecAttr (*codecInfo, "BitRate", divxbitrate);
      Creators::SetCodecAttr (*codecInfo, "Crispness", crispness);
      Creators::SetCodecAttr (*codecInfo, "KeyFrames", keyframes);
    }
#else
  IVideoEncoder::SetExtendedAttr (codec, "BitRate", divxbitrate);
#endif

  BITMAPINFOHEADER bh;
  memset (&bh, 0, sizeof (BITMAPINFOHEADER));
  bh.biSize = sizeof (BITMAPINFOHEADER);
  bh.biWidth = width;
  bh.biHeight = height;
  bh.biPlanes = 1;
  bh.biBitCount = 24;
  bh.biCompression = 0;
  bh.biSizeImage = width * height * 3;

  unsigned char *scaledbuffer = 0;
  CImage *im;
  if (!(oheight || owidth))
    im =
      new CImage ((BitmapInfo *) & bh,
		  framebuffer + (cliplines - clipoffset) * width * 3, false);
  else
    {
      if (oheight)
	bh.biHeight = oheight;
      if (owidth)
	bh.biWidth = owidth;
      bh.biSizeImage = bh.biHeight * bh.biWidth * 3;
      scaledbuffer = new unsigned char[bh.biHeight * bh.biWidth * 3];
      im = new CImage ((BitmapInfo *) & bh, scaledbuffer, false);
    }

  IAviVideoWriteStream *stream =
    avifile->AddVideoStream (fccHandler, &bh, (int) (1000000 / framerate));

  // It appears to be ok to cast framerate divided 
  // by 1000000 to int, at least for framerate = 30
  // I use xawtv to generate movies in mjpeg 
  // which avifile can handle...might try setting framerate
  // to something stupid, like 11 or 1...later...
  // 11 seems to work just fine.

  stream->SetQuality (8000);	// Should be an option I think? 
				// But I cannot see that it does
				// anything.
  stream->Start ();

  WAVEFORMATEX fmt;
  memset (&fmt, 0, sizeof (WAVEFORMATEX));
  fmt.wFormatTag = 1;
  fmt.nChannels = (achan > 2 ? 2 : achan);
  fmt.nSamplesPerSec = rate;
  fmt.nAvgBytesPerSec = (achan > 1 ? rate : rate * 2);
  fmt.nBlockAlign = (achan > 1 ? 2 : 1);
  fmt.wBitsPerSample = 16;
  fmt.cbSize = sizeof (WAVEFORMATEX);

  if (achan > 0 && mp3bitrate == -1)
    switch (rate)
      {
      case 48000:
	mp3bitrate = (achan > 1 ? 160 : 80 * achan);
	break;
      case 44100:
	mp3bitrate = (achan > 1 ? 112 : 56 * achan);
	break;
      case 22050:
	mp3bitrate = (achan > 1 ? 64 : 32 * achan);
	break;
      default:
	printf ("unknown sample rate %i Hz, please specify MP3 bitrate\n",
		rate);
	exit (1);
      }

  IAviAudioWriteStream *astream = 0;

  if (achan > 0)
    {
      astream = avifile->AddAudioStream (0x55, &fmt, (mp3bitrate * 1000) / 8);
      astream->Start ();
    }

  int samples_read = 0;

  int totalframes = 0;
  int totalframes_allfiles = frames;

  struct timeval tv;
  struct timezone tz = { 0, 0 };
  gettimeofday (&tv, &tz);
  long startsec = tv.tv_sec;
  long startusec = tv.tv_usec;
  double fps = 0;
  for (int j = 0; j < numfiles; j++)
    {
      if (j)
	{
	  mpeg3_close (file);
	  printf ("opening MPG %s\n", inputfiles[j]);
	  file = mpeg3_open (inputfiles[j]);
	  frames = mpeg3_video_frames (file, vistream);
	  totalframes_allfiles += frames;
	}
      if (startframe > 0)
	{
	  printf ("Skipping %d frames\n", startframe);
	  frames -= startframe;
	  totalframes_allfiles -= startframe;
	  mpeg3_set_frame (file, startframe, vistream);
	  mpeg3_set_sample (file, startframe * sampsperframe, austream);
	  startframe = 0;
	}
      frames -= skipvframes;
      totalframes_allfiles -= skipvframes;
      if (skipvframes > 0)
	{
	  mpeg3_set_frame (file, startframe + skipvframes, vistream);
	}
      else if (skipvframes < 0)
	{
	  frames -= skipvframes;
	}
      skipvframes = 0;

      for (int i = 0; i < frames - 15; i++, totalframes++)
	{
	  if (endframe > 0 && totalframes >= endframe)
	    goto finished;

	// if(totalframes%10 == 0) printf("Encoding frame
	// %d...\n",totalframes);
	  gettimeofday (&tv, &tz);
	  fps =
	    totalframes / ((tv.tv_sec + tv.tv_usec / 1000000.0) -
			   (startsec + startusec / 1000000.0));
	  printf ("\rEncoding frame %d of %d, %.1f frames per sec    ",
		  totalframes, totalframes_allfiles, fps);
	  fflush (stdout);

	  if (mpeg3_read_frame
	      (file, rowptr, 0, 0, width, height + cliplines - clipoffset,
	       width, height + cliplines - clipoffset, MPEG3_BGR888,
	       vistream))
	    {
	      printf ("error reading frame %d\n", i);
	      delete avifile;
	      exit (1);
	    }
	  memcpy (framebuffer, extrabuffer, width * 3);

	  if (achan > 0)
	    {
	      int downmix51 = 0;
	      int result = 0;
	      int16_t *input_end;
	      int len = sampsperframe;

	      char *buffer = (char *) audiobufferint;
	      int16_t *output_ptr = NULL;
	      int16_t *input_ptr = (int16_t *) read_buffer;
	      input_end = input_ptr + len;

		// int assetChannels = mpeg3_audio_channels(file,
		// austream);
	      int assetChannels = achan;
	      if (assetChannels > 2)
		{
		  assetChannels = 2;
		}

	      if (achan > assetChannels)
		{
		  downmix51 = 1;
		  for (int channel = 0; channel < assetChannels; channel++)
		    {
		      output_ptr = (int16_t *) buffer + channel;
		      int16_t *output_end =
			(int16_t *) output_ptr + len * assetChannels;
		      while (output_ptr < output_end)
			{
			  *output_ptr++ = 0;
			}
		    }
		}

	      int total_channels = (downmix51 ? achan : assetChannels);

	      for (int channel = 0;
		   channel < total_channels && !result; channel++)
		{
		  if (channel == 0)
		    result = mpeg3_read_audio (file,
					       0,
					       read_buffer,
					       channel, len, austream);
		  else
		    result = mpeg3_reread_audio (file,
						 0,
						 read_buffer,
						 channel, len, austream);

		// Prepare for next channel
		// Incompatible with percentage seeking
		//              if(channel < total_channels - 1 &&
		// !result) mpeg3_set_sample(file, 
		// mpeg3_get_sample(file, 0) - len,
		// austream);

		  // Interlace the audio for the sound driver.
		  if (downmix51)
		    {
		      input_ptr = (int16_t *) read_buffer;
		      input_end = input_ptr + len;

		      switch (channel)
			{
			case 0:
			  output_ptr = (int16_t *) buffer;
			  break;	// Left
			case 1:
			  output_ptr = (int16_t *) buffer;
			  break;	// Center
			case 2:
			  output_ptr = (int16_t *) buffer + 1;
			  break;	// Right
			case 3:
			  output_ptr = (int16_t *) buffer;
			  break;	// Left
			case 4:
			  output_ptr = (int16_t *) buffer + 1;
			  break;	// Right
			}

		      if (channel == 1)
			while (input_ptr < input_end)
			  {
			    *output_ptr++ += *input_ptr;
			    *output_ptr++ += *input_ptr++;
			  }
		      else
			while (input_ptr < input_end)
			  {
			    *output_ptr += *input_ptr++;
			    output_ptr += assetChannels;
			  }
		    }
		  else
		    {
		      output_ptr = (int16_t *) buffer + channel;
		      input_ptr = (int16_t *) read_buffer;
		      input_end = input_ptr + len;

		      while (input_ptr < input_end)
			{
			  *output_ptr = *input_ptr++;
			  output_ptr += assetChannels;
			}
		    }
		}

	      if (achan > 1)
		astream->AddData (audiobufferint, sampsperframe * 2);
	      else
		astream->AddData (audiobufferint, sampsperframe);
	    }


	//printf("framebuffer = %d %d
	//%d\n",framebuffer[0],framebuffer[1],framebuffer[2]);
	//printf("Writing frame %d...\n",i);
	  if (oheight || owidth)	//(y+cliplines-clipoffset)
	    {
	      ScaleX = (float) bh.biWidth / width;
	      ScaleY = (float) bh.biHeight / height;

	      for (y = 0; abs ((int) y) < abs (bh.biHeight); y++)	// obscure gcc error needs abs jpc
		{
		  NewY1 =
		    (unsigned int) (y / ScaleY + cliplines -
				    clipoffset) * /*Image->Bps */ width * 3;
		  for (x = 0; abs ((int) x) < abs (bh.biWidth); x++)	// obscure gcc error needs abs jpc
		    {
		      t1 = x / (double) bh.biWidth;
		      t4 = t1 * bh.biWidth;
		      t2 = t4 - (unsigned int) (t4);
		      ft = t2 * M_PI;
		      f = (1.0 - cos (ft)) * .5;
		      NewX1 = ((unsigned int) (t4 / ScaleX)) * 3;
		      NewX2 = ((unsigned int) (t4 / ScaleX) + 1) * 3;

		      Size = y * bh.biWidth * 3 /*Scaled->Bps */  + x * 3;
		      for (c = 0; c < 3; c++)
			{
			  x1 = framebuffer[NewY1 + NewX1 + c];
			  x2 = framebuffer[NewY1 + NewX2 + c];
			  //printf ("X1: %d, X2: %d, Size: %d\n", NewY1 + NewX1 + c, NewY1 + NewX2 + c, width * height *3);
			  scaledbuffer[Size + c] =
			    (unsigned char) ((1.0 - f) * x1 + f * x2);
			  //printf("Scale: %d, Size: %d\n", Size + c, bh.biWidth*bh.biHeight *3 );
			}
		    }
		}
	    }
	  if (blur)
	    {
	      im->ToYUV ();
	      im->Blur (blur);
	    }
	  stream->AddFrame (im);

	  samples_read += sampsperframe;
	//printf("samples: %d, samples_read: %d\n",samples,samples_read);
	}
    }
finished:
  stream->Stop ();
  delete avifile;
  mpeg3_close (file);
}
